<?php
/**
 * Yado MediaWiki extension  version 1.00
 *	for details please see: https://kimagurenote.net/kn/Yado
 *
 *	required MediaWiki 1.28 or above.
 *	https://www.mediawiki.org/wiki/Manual:LinkRenderer.php
 *
 * Copyright (c) 2019 Kimagurenote https://kimagurenote.net/
 * License: Revised BSD license http://opensource.org/licenses/BSD-3-Clause
 *
 * Function:
 *	Mediawiki extension that make hotel links for Rakuten reservation site.
 *
 * Usage:
 *	&lt;yado&gt;free keyword&lt;/yado%gt; on your wiki page.
 *
 * History:
 * 2019.05.26 Version 1.00
 *	1st test
 *
 * @file
 * @ingroup Extensions
 * @author Kimagurenote
 * @copyright © 2019 Kimagurenote
 * @license The BSD 3-Clause License
 */

if( !defined( 'MEDIAWIKI' ) ) {
	echo( "This file is an extension to the MediaWiki software and cannot be used standalone.\n" );
	die( 1 );
}

$wgExtensionCredits['parserhook'][] = array(
	'path'           => __FILE__,
	'name'           => 'Yado',
	'author'         => array(
		'[https://kimagurenote.net/ Kimagurenote]'
	),
	'url'            => 'https://kimagurenote.net/kn/Yado',
	'version'        => '1.00',
	'license-name'   => 'BSD',
	'descriptionmsg' => 'yado-desc'
);

/* Internationalization */
$wgMessagesDirs['Yado'] = __DIR__ . '/i18n';

/* default values */
define( 'YADO_HOOKID', 'yado' );
define( 'YADO_CLASS_DIV', 'yado' );

/* query type identifier */
define( 'YADO_SELECTTYPE_TOP', 'top' );
define( 'YADO_SELECTTYPE_RANDOM', 'random' );

/* operation parameter */
define( 'YADO_NAMEMAXLEN', -1 );
define( 'YADO_SPECIALMAXLEN', -1 );

/* Rakuten Travel keyword hotel search API URL (json)
 * https://webservice.rakuten.co.jp/api/keywordhotelsearch/
 */
$wgYadoRakutenAPIURL = 'https://app.rakuten.co.jp/services/api/Travel/KeywordHotelSearch/20170426';

/* default tag classes */
$wgYadoClass['div'] = '';
$wgYadoClass['img'] = 'yadoimg';
$wgYadoClass['text'] = 'yadotext';
$wgYadoClass['hotelName'] = 'yadoname';
$wgYadoClass['hotelSpecial'] = 'yadospecial';
$wgYadoClass['nearestStation'] = 'yadostation';
$wgYadoClass['rate'] = 'yado-rate';
$wgYadoClass['rate-mark'] = 'yado-rate-mark';
$wgYadoClass['rate-mark-front'] = 'front';
$wgYadoClass['rate-mark-back'] = 'back';
$wgYadoClass['rate-score'] = 'yado-rate-score';
$wgYadoClass['review'] = 'yado-review';
$wgYadoRate = false;
$wgYadoRateSize = 16;

/* accepted parameters for pass to API */
$wgYadoAcceptArgs = array( 'sort' );


/* MediaWiki set hook */
$wgHooks['ParserFirstCallInit'][] = 'wfYadoSetHook';

function wfYadoSetHook( $parser ) {
	$parser->setHook( YADO_HOOKID, 'wfRenderYado' );
	return true;
}

/* Main process - The callback function for embed shopping link
 *
 */
function wfRenderYado( $value, $argv, $parser ) {
	global $wgYadoRakutenAPIURL, $wgYadoRakutenAppID, $wgYadoRakutenAffID, $wgYadoAcceptArgs, $wgYadoClass, $wgYadoMaxLen, $wgYadoStationBaseURL, $wgYadoRate, $wgYadoRateSize;

	/* search keyword */
	$value = mb_convert_encoding(trim($value), 'UTF8', 'ASCII,JIS,UTF-8,EUC-JP,SJIS-WIN');

	/* require apiurl, appid and keyword(value) */
	if( empty($wgYadoRakutenAPIURL) || empty($wgYadoRakutenAppID) || empty($value) ) {
		/* for debug */	wfDebugLog( YADO_HOOKID, "wfRenderYado(): required parameter(s) empty: $wgYadoRakutenAPIURL, $wgYadoRakutenAppID, $value.\n" );
		return '';
	}

	/* generate ID (for identify tag) */
	$tagid = YADO_HOOKID . '-' . time();

	/* generate API URL */
	$apiurl = $wgYadoRakutenAPIURL . '?applicationId=' . $wgYadoRakutenAppID;

	/* Rakuten affiliate ID (option)
	 * https://webservice.rakuten.co.jp/account_affiliate_id/
	 */
	if( !empty($wgYadoRakutenAffID) ) {
		$apiurl .= '&affiliateId=' . urlencode($wgYadoRakutenAffID);
	}

	/* common parameters */
	$apiurl .= '&format=json&formatVersion=2';

	/* select type (default: top) */
	$selecttype = @$argv['selecttype'];
	if ( strcasecmp($selecttype, YADO_SELECTTYPE_RANDOM) == 0 ) {
		/* ランダムで表示 */
		$selecttype = YADO_SELECTTYPE_RANDOM;
		srand(time());
	} else {
		/* 最も上位の検索結果を表示(default) */
		$selecttype = YADO_SELECTTYPE_TOP;
		$apiurl .= '&hits=1';
	}

	/* keyword */
	$apiurl .= '&keyword=' . urlencode($value);

	/* more parameters if accepted (option) */
	if( !empty($args = @$argv) ) {
		foreach( $wgYadoAcceptArgs as $v ) {
			if( !empty($args[$v]) ) {
				$apiurl .= '&' . $v . '=' . urlencode($args[$v]);
				unset($args[$v]);
			}
		}
	}

	/* call API */
	if( ($retapi = file_get_contents($apiurl)) === false ) {
		/* for debug */	wfDebugLog( YADO_HOOKID, "wfRenderYado(): can't call api $apiurl.\n" );
		return '';
	}

	/* for debug */	wfDebugLog( YADO_HOOKID, "wfRenderYado(): API call success.\n$apiurl\n" );

	/* decode JSON */
	if( is_null( $resapi = json_decode( mb_convert_encoding( $retapi, 'UTF8', 'ASCII,JIS,UTF-8,EUC-JP,SJIS-WIN'), true ) ) ) {
		/* for debug */	wfDebugLog( YADO_HOOKID, "wfRenderYado(): can't decode json.\n" . $resapi . "\n" );
		return '';
	}

	/* no result */
	if( $resapi['pagingInfo']['recordCount'] < 1 ) {
		/* for debug */	wfDebugLog( YADO_HOOKID, "wfRenderYado(): no result.\n" . var_export($resapi, TRUE) . "\n" );
		return '';
	}

	/* get result, render start */
	if($selecttype == YADO_SELECTTYPE_RANDOM) {
		$n = rand($resapi['pagingInfo']['first'], $resapi['pagingInfo']['last']) - 1;
	} else {
		$n = 0;
	}
	$res0 = $resapi['hotels'][$n][0]['hotelBasicInfo'];

	/* illegal result */
	if( empty($res0['hotelName']) ) {
		/* for debug */	wfDebugLog( YADO_HOOKID, "wfRenderYado(): illegal result.\n" . var_export($resapi, TRUE) . "\n" );
		return '';
	}

	/* generate embed HTML tags */
	$ret = '<div class="' . YADO_CLASS_DIV;
	if( !empty($wgYadoClass['div']) ) {
		$ret .= ' ' . $wgYadoClass['div'];
	}
	$ret .= '" style="display: table;"><a href="' . $res0['hotelInformationUrl'] . '" target="_blank" rel="nofollow">';
	$ret .= '<img src="' . $res0['hotelImageUrl'] . '"';
	if( !empty($wgYadoClass['img']) ) {
		$ret .= ' class="' . $wgYadoClass['img'] . '"';
	}
	$ret .= ' style="display: table-cell;" /></a><div';
	if( !empty($wgYadoClass['text']) ) {
		$ret .= ' class="' . $wgYadoClass['text'] . '"';
	}
	$ret .= ' style="display: table-cell; vertical-align: top;">';
	/* hotel name */
	$name = $res0['hotelName'];
	$maxlen = $wgYadoMaxLen['hotelName'] ? $wgYadoMaxLen['hotelName'] : YADO_NAMEMAXLEN;
	$ret .= '<p';
	if( !empty($wgYadoClass['name']) ) {
		$ret .= ' class="' . $wgYadoClass['name'] . '"';
	}
	$ret .= '><a href="' . $res0['hotelInformationUrl'] . '" target="_blank" rel="nofollow">';
	if( $maxlen < 0 || strlen($name) <= $maxlen ) {
		$ret .= htmlspecialchars($name);
	} else {
		$ret .= htmlspecialchars(mb_strcut($name, 0, $maxlen) . '…');
	}
	$ret .= '</a></p>';

	/* hotel special comment */
	$text = $res0['hotelSpecial'];
	$maxlen = $wgYadoMaxLen['hotelSpecial'] ? $wgYadoMaxLen['hotelSpecial'] : YADO_SPECIALMAXLEN;
	$ret .= '<p';
	if( !empty($wgYadoClass['hotelSpecial']) ) {
		$ret .= ' class="' . $wgYadoClass['hotelSpecial'] . '"';
	}
	$ret .= '>';
	if( $maxlen < 0 || strlen($text) <= $maxlen ) {
		$ret .= htmlspecialchars($text);
	} else {
		$ret .= htmlspecialchars(mb_strcut($text, 0, $maxlen) . '…');
	}
	$ret .= '</p>';

	/* nearestStation */
	if( !empty($text = trim($res0['nearestStation'])) ) {
		$text .= '駅';
		$ret .= '<p';
		if( !empty($wgYadoClass['nearestStation']) ) {
			$ret .= ' class="' . $wgYadoClass['nearestStation'] . '"';
		}
		$ret .= '>' . wfMessage( 'yado-nearesttation' )->inContentLanguage()->escaped();
		$title = Title::newFromText( $text );
		if( $title->exists() ) {
			$linkRenderer = $parser->getLinkRenderer();
			$ret .= $linkRenderer->makeLink( new TitleValue( NS_MAIN, $text ) );
		} elseif( !empty($wgYadoStationBaseURL) ) {
			$ret .= '<a href="' . $wgYadoStationBaseURL . urlencode($text) . '" target="_blank">' . htmlspecialchars($text) . '</a>';
		} else {
			$ret .= htmlspecialchars($text);
		}
		$ret .= '</p>';
	}

	/* rate and review */
	if( $wgYadoRate && $res0['reviewAverage'] ) {
		$ret .= '<div style="display: table;"><div class=' . $wgYadoClass['rate'] . ' style="display: table-cell; width: ' . $wgYadoRateSize * 5 . 'px;"><a href="' . $res0['reviewUrl'] . '" title="' . wfMessage( 'yado-review-title', $name, $res0['reviewCount'] )->inContentLanguage()->escaped() . '" target="_blank" rel="nofollow">';
		$ret .= '<pre class="' . $wgYadoClass['rate-mark'] . '"><span class="' . $wgYadoClass['rate-mark-back'] . '"> </span><span class="' . $wgYadoClass['rate-mark-front'] . '" style="width: ' . intval($wgYadoRateSize * $res0['reviewAverage'] + 0.5) . 'px;"> </span></pre>';
		$ret .= '<p class="' . $wgYadoClass['rate-score'] . '">' . htmlspecialchars($res0['reviewAverage']) . '</p></a></div><div class="' . $wgYadoClass['review'] . '" style="display: table-cell;">';
		if( !empty($res0['userReview']) ) {
			$ret .= '<p>' . htmlspecialchars($res0['userReview']) . '</p>';
		}
		$ret .= '</div></div>';
	}

	$ret .= "</div></div>\n";

	/* for debug */	wfDebugLog( YADO_HOOKID, "wfRenderYado() complete: \n$ret\n" );

	return $ret;
}

?>
